// CACrtsIntWW2DDatum - Integer data storage for WireWorld rules set
// Implementation file
/////////////////////////////////////////////////////////////////////////////

// CADLib includes
#include <AfxDpnds.h>
#include <Common.h>
#include <CADLibRes.h>
#include <Dlg\\Dlg.h>
#include <Parameter.h>
#include <Dlg\\Dialogs.h>
#include <TypedParameters.h>
#include <Component.h>
#include <Zone.h>
#include <WorkStationInfo.h>
#include <SmartBuffer.h>
#include <Network.h>
#include <Datum.h>
#include <Grid.h>
#include <Metrics.h>
#include <Cooperation.h>
#include <RulesDev.h>
#include <BasicCrts2DDatum.hpp>

#include "IntChoiceDlg.cpp"

// Component dependent includes
#include "Resource.h"
#include "CrtsIntWW2D.h"

// Implements the autentification function
COMPATIBLE_DATUM(1.1)
// Implements the creation/destruction functions
DATUM_COMPONENT(CACrtsIntWW2DDatum)

CACrtsIntWW2DDatum::CACrtsIntWW2DDatum():
	p_iMin("Minimum", "Minimal value of cells data.", 0, 0, 3, this),
	p_iMax("Maximum", "Maximal value of cells data.", 3, 0, 3, this),
	p_cInitial("Initial", "Initial value of new data elements", 0, "Background|Wire|Electron Tail|Electron Head", this)
{
	p_bDouble = *(new ParamBool("Double", "If true then there will be two copies of data storage for two consecutive steps and they will be switched between the iterations", true, this));
	p_col[WW_CELL_BACK] = new ParamColor("Color for Background cells", "Color for cells in Background state", RGB(0, 0, 0), this);
	p_col[WW_CELL_WIRE] = new ParamColor("Color for Wire cells", "Color for cells in Wire state", RGB(0xff, 0xcc, 0), this);
	p_col[WW_CELL_TAIL] = new ParamColor("Color for Electron Tail cells", "Color for cells in Electron Tail state", RGB(0, 0, 0xff), this);
	p_col[WW_CELL_HEAD] = new ParamColor("Color for Electron Head cells", "Color for cells in Electron Head state", RGB(0xff, 0, 0), this);
	Const=p_cInitial.Get();
}

CACrtsIntWW2DDatum::~CACrtsIntWW2DDatum()
{
	delete p_col[WW_CELL_BACK];
	delete p_col[WW_CELL_WIRE];
	delete p_col[WW_CELL_TAIL];
	delete p_col[WW_CELL_HEAD];
}

COLORREF CACrtsIntWW2DDatum::GetCellColor(CACell c)
{
	int state;
	register CACell x = 0, y = 0, z = 0;
	GetUnion()->Metrics->ToAxis(c, x, y, z);

	if (p_bDouble.Get())
	{
		state = ((bComputation)?(*pDataOld):(*pData))[(unsigned int)(y - zfull.a2)][(unsigned int)(x - zfull.a1)];
	}
	else
	{
		state = (*pData)[(unsigned int)(y - zfull.a2)][(unsigned int)(x - zfull.a1)];
	}
	if (state > p_iMax.Get() || state < p_iMin.Get()) state = 0;
	return p_col[state]->Get();
}

void CACrtsIntWW2DDatum::Set(CACell c, int value)
{
	if (value > p_iMax.Get()) value = p_iMax.Get();
	if (value < p_iMin.Get()) value = p_iMin.Get();
	CABasicCrts2DDatum<int, CIntChoiceDlg, IDD_CHOICE>::Set(c, value);
}

bool CACrtsIntWW2DDatum::Query(CWnd* pParent, long x, long y, CACell c, bool selected)
{
	HINSTANCE hInst = GetCADResHandle();
	NOT_NULL(hInst);

	// Lets get resource handle
	HRSRC hSrc = FindResource(hInst, MAKEINTRESOURCE(IDD_CHOICE), RT_DIALOG);
	NOT_NULL(hSrc);

	// Lets get dialogs handle and tune it
	HGLOBAL hDlg = LoadResource(hInst, hSrc);
	NOT_NULL(hDlg);

	// Create dialog window
	DLGTEMPLATE* pTemplate = (DLGTEMPLATE*) LockResource(hDlg);

	TCHAR str[_MAX_PATH];
	UINT u;

	if (selected)
	{
		strcpy(str, "Selected");
		u = GetUnion()->Grid->SelCells.size();
	}
	else
	{
		GetUnion()->Metrics->ToString(c, str);
		u = 1;
	}

	int** b = new int* [u];

	CACell cx = 0, cy = 0, cz = 0;
	if (selected)
	{
		CACellsList::iterator it = GetUnion()->Grid->SelCells.begin();
		for (UINT ui = 0; ui < u; ui++)
		{
			c = *it;
			GetUnion()->Metrics->ToAxis(c, cx, cy, cz);
			it++;

			b[ui] = &(*pData)[(unsigned int)(cy - zfull.a2)][(unsigned int)(cx - zfull.a1)];
		}
	}
	else
	{
		GetUnion()->Metrics->ToAxis(c, cx, cy, cz);
		b[0] = &(*pData)[(unsigned int)(cy - zfull.a2)][(unsigned int)(cx - zfull.a1)];
	}

	CIntChoiceDlg* pDlg = new CIntChoiceDlg(pParent, x, y, b, u, str, p_iMin.Get(), p_iMax.Get(), "Background|Wire|Electron Tail|Electron Head");

	pDlg->InitModalIndirect(pTemplate, pDlg->m_pParent);

	// Run dialogs thread
	AfxBeginThread(DoQueryModal,pDlg);

	return true;
}

void CACrtsIntWW2DDatum::ParameterChanged(Parameter* pPar)
{
	if (pPar == NULL) {
		// Does p_iMin and p_iMax actually are minimum and maximum
		if (p_iMin.Get() == p_iMax.Get()) p_iMin.Set(p_iMax.Get() / 2);

		if (p_iMin.Get() > p_iMax.Get())
		{
			int i = p_iMin.Get();
			p_iMin.Set(p_iMax.Get());
			p_iMax.Set(i);
		}

		// If initial value is in the accessible range
		if (p_cInitial.Get() > p_iMax.Get()) p_cInitial.Set((unsigned char)p_iMax.Get());
		if (p_cInitial.Get() < p_iMin.Get()) p_cInitial.Set((unsigned char)p_iMin.Get());

		// Change parents Const value
		Const = p_cInitial.Get();

		// Check the range of values
		for (UINT i = 0; i < uiHeight; i++)
		{
			for (UINT j=0;j<uiWidth;j++)
			{
				if ((*pData)[i][j] < p_iMin.Get()) (*pData)[i][j] = p_iMin.Get();
				if ((*pData)[i][j] > p_iMax.Get()) (*pData)[i][j] = p_iMax.Get();
			}
		}

	}

	if (pPar == &p_cInitial)
	{
		// If initial value is in the accessible range
		if (p_cInitial.Get() > p_iMax.Get()) p_cInitial.Set((unsigned char)p_iMax.Get());
		if (p_cInitial.Get() < p_iMin.Get()) p_cInitial.Set((unsigned char)p_iMin.Get());

		// Change parents Const value
		Const = p_cInitial.Get();
		return;
	}

	if (pPar == &p_iMin || pPar == &p_iMax)
	{
		// Does p_iMin and p_iMax actually are minimum and maximum
		if (p_iMin.Get() == p_iMax.Get()) p_iMin.Set(p_iMax.Get() / 2);

		if (p_iMin.Get() > p_iMax.Get())
		{
			int i = p_iMin.Get();
			p_iMin.Set(p_iMax.Get());
			p_iMax.Set(i);
		}

		// If initial value is in the accessible range
		if (p_cInitial.Get() > p_iMax.Get()) p_cInitial.Set((unsigned char)p_iMax.Get());
		if (p_cInitial.Get() < p_iMin.Get()) p_cInitial.Set((unsigned char)p_iMin.Get());

		// Check the range of values
		for (UINT i = 0; i < uiHeight; i++)
		{
			for (UINT j = 0; j < uiWidth; j++)
			{
				if ((*pData)[i][j] < p_iMin.Get()) (*pData)[i][j] = p_iMin.Get();
				if ((*pData)[i][j] > p_iMax.Get()) (*pData)[i][j] = p_iMax.Get();
			}
		}

		// Update grid
		GetUnion()->Grid->Paint(false,false);
		return;
	}

	// Execute CABasicCrts2DDatum<type,dlg,idd>'s ParameterChanged()
	CABasicCrts2DDatum<int, CIntChoiceDlg, IDD_CHOICE>::ParameterChanged(pPar);
}

bool CACrtsIntWW2DDatum::Load(IOStream* st)
{
	bool b = CABasicCrts2DDatum<int, CIntChoiceDlg, IDD_CHOICE>::Load(st);
	if (!b) return false;

	// Check the range of values
	for (UINT i = 0; i < uiHeight; i++)
	{
		for (UINT j = 0; j < uiWidth; j++)
		{
			if ((*pData)[i][j] < p_iMin.Get()) (*pData)[i][j] = p_iMin.Get();
			if ((*pData)[i][j] > p_iMax.Get()) (*pData)[i][j] = p_iMax.Get();
		}
	}
	return true;
}
